%% 22/04/2020 - A script to calculate quantisation noise in multiband concurrent receivers and produce performance curves. Created by Dr Stephen Henthorn, used in Concurrent Multiband Direct RF Sampling Receivers. Please attribute when used for further work (CC BY-NC 4.0)
clear all
close all

%% Key parameters
%ADC parameters
b=8; %ADC bits
fs=10e9; %ADC sampling frequency
sigj=360e-15; %ADC sampling jitter standard deviation (s^-1)
Vmin=0.04; %Minimum Vpp for ADC (V)
adcBW=4e9; %ADC RF bandwidth

%Receiver parameters
K=1; %Number of bands
fc=[888e6,1.92e9,2.52e9,3.45e9]; %Centre frequency of each band
BW=20e6*ones(1,4); %Bandwidth of each band (vector length K)
R=50; %Receiver impedance
FdB=[3.1,3.5,3.6,4.2];
Glna2dB=[21.4,21.2,20.9,20.6]; %LNA2 gain for each band (dB)
Flna2dB=[1.33,1.44,1.50,1.48]; %LNA2 noise figures for each band (dB)
GsplitdB=[-6.3,-6.3,-6.3,-6.6]; %Splitter/combiner gain for each band (dB)
GsawdB=[-1.4,-1.6,-1.3,-3.6]; %SAW filter gain for each band (dB)
GdB=Glna2dB+GsplitdB+GsawdB+GsplitdB+Glna2dB; %Gain of RF front-end for each band

%Other values
SrfThermal=1.38e-23*290; %RF noise density (kT)
SrfInput=SrfThermal; %Change where input noise is greater than general thermal noise (eg. due to interference)
z=[1.645,1.960,2.575,4]; %z-values for [0.90,0.95,0.99] CDF passing through ADC
Cind=3; %Which value of C (and so z) to use

%% Define power ranges
BoI=2; %Which of K bands is the one under study?
pBoIdBm=-100:1:-50; %Signal power at receiver input for BoI (dBm) (can be a vector)
if K==1
    pOBdBm=-9999;
else
    pOBdBm=-50; %Signal power at receiver input for other bands (dBm) (scalar only)
end

%% RF Noise Calculations
%Preliminary
Lq=2^b; %Number of levels
pBoI=10.^((pBoIdBm-30)/10); %dB to linear
pOB=10^((pOBdBm-30)/10);
G=10.^(GdB./10);
F=10.^(FdB./10);
Glna2=10.^(Glna2dB./10);
Flna2=10.^(Flna2dB./10);

%RF noise power calculations
NrfT=SrfInput*BW(BoI); %RF in-band noise power at receiver input
NrfInBand=NrfT*G(BoI)*F(BoI); %RF noise power in-band at ADC input is product of input noise, noise factor and gain for the band
NrfInBanddBm=10*log10(NrfInBand)+30; %Convert to dBm
NDensrfOutOfBand=SrfThermal*Glna2(BoI)*Flna2(BoI); %RF noise density out-of-band (ie. amplified by 2nd LNA)

%% Digital Noise Calculations
%RF noise folding
nFolds=floor(2*adcBW/fs); %Number of folds
if nFolds>0 %If true, subband. If false, Nyquist
    NrfOOBFolded=NDensrfOutOfBand*BW(BoI)*(nFolds-1); %RF out-of-band noise folded into band
    NrfOOBFoldeddBm=10*log10(NrfOOBFolded)+30; %Store in dBm for reference
else
    NrfOOBFolded=0; %No folding, no OOB noise added in
end
pIn=zeros(K,length(pBoIdBm)); %Preallocate array for powers at receiver input for each band at each test point
for k=1:4 %Fill array
    if k==BoI
        pIn(k,:)=pBoI;
    else
        pIn(k,:)=pOB*ones(1,length(pBoI));
    end
end
pADC=G*pIn; %Multiply powers in by gain of each band, and sum them across K bands

%Quantisation noise
qTerm1=BW(BoI)./(6*R*Lq^2*fs);%Static term in quantisation noise calculation

Vpp=4*z(Cind)*sqrt(R*pADC); %Calculate peak-to-peak voltage required
Vpp(Vpp<Vmin)=Vmin; %Replace Vpp values less than Vmin with Vmin

Nq=Vpp.^2.*qTerm1;
NqdBm=10*log10(Nq)+30; %dB

%Jitter noise
if K==1
    Pnj=pADC*(2*pi*fc(BoI)*sigj).^2;
else
    for kInd=1:K
        pADCInd(kInd,:)=pIn(kInd,:)*G(kInd); %Signal power at ADC input in each band
        PnjInd(kInd,:)=pADCInd(kInd,:)*(2*pi*fc(kInd)*sigj)^2; %Noise density introduced by jitter of each band
    end
    Pnj=sum(PnjInd); %Total noise density is sum of noise due to all bands
end

Nj=Pnj*BW(BoI)/fs; %Noise power in-band for each band
NjdBm=10*log10(Nj)+30; %Calculate in dBm for reference

%% Total and performance curves
%Total
Ntot=Nq+NrfInBand+NrfOOBFolded+Nj; %Sum noise components
NtotdBm=10*log10(Ntot)+30;

%% Display results
disp('%%% ------------------------------ %%%')
disp(['For Band ' num2str(BoI) ':'])
disp(['Noise power in-band due to RF noise: ' num2str(NrfInBanddBm) ' dBm'])
disp(['Maximum noise power due to quantisation noise: ' num2str(max(NqdBm)) ' dBm'])
disp(['Maximum total noise power after reception: ' num2str(max(NtotdBm)) ' dBm'])
disp('%%% ------------------------------ %%%')

%Performance curves
powADCdBm=10*log10(pBoI*G(BoI));
snr=(pBoI*G(BoI))./Ntot; %Signal power to noise power ratio
snrdB=10*log10(snr);
evm=1./(sqrt(snr*2)); %Approximate EVM based on SNR

%Plots
figure(1)
plot(pBoIdBm,evm*100,'-')
axis([min(pBoIdBm) max(pBoIdBm) 0 100])
xlabel('Signal input power (dBm)')
ylabel('EVM (%)')